//+------------------------------------------------------------------+
//|                                                          HMA.mq4 |
//|                      Copyright  2004, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#property  copyright "Copyright  2004, MetaQuotes Software Corp."
#property  link      "http://www.metaquotes.net/"
//---- indicator settings
#property indicator_chart_window 
#property indicator_buffers 3 
#property indicator_color1 Magenta 
#property indicator_color2 Aqua
#property indicator_color3 Tomato 
#property indicator_width1 4
#property indicator_width2 4
#property indicator_width3 4 
//---- indicator parameters
extern int     Price          = 0;
extern int     HMA_Period     = 20;
extern int     ColorMode      = 1;
extern int     ColorBarBack   = 1;
extern int     AlertMode      = 1;   //POP UP MESSAGE (0-off,1-on) 
extern int     WarningMode    = 1;   //Sound Warning switch(0-off,1-on) 
extern int     SoundOn        = 1;   //Sound switch(0-off,1-on) 
extern int     EmailOn        = 0;   // email on when = 1;
extern string  SoundFileName  = "pirate.wav";
//---- indicator buffers
double     ind_buffer0[];
double     Uptrend[];
double     Dntrend[];
double     ind_buffer1[];
double     trend[];

int        draw_begin0;
bool       UpTrendAlert=false, DownTrendAlert=false;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicator buffers mapping
   IndicatorBuffers(5);
   SetIndexBuffer(0,ind_buffer0);
   SetIndexBuffer(1,Uptrend);
   SetIndexBuffer(2,Dntrend);
   SetIndexBuffer(3,ind_buffer1);
   SetIndexBuffer(4,trend);  
//---- drawing settings
   SetIndexStyle(0,DRAW_LINE);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexStyle(2,DRAW_LINE);
   draw_begin0=HMA_Period+MathFloor(MathSqrt(HMA_Period));
   SetIndexDrawBegin(0,draw_begin0);
   SetIndexDrawBegin(1,draw_begin0);
   SetIndexDrawBegin(2,draw_begin0);
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS)+1);
//---- name for DataWindow and indicator subwindow label
   IndicatorShortName("HMA("+HMA_Period+")");
   SetIndexLabel(0,"HullMA");
   SetIndexLabel(1,"HullMA Uptrend");
   SetIndexLabel(2,"HullMA Dntrend");
//---- initialization done
   return(0);
  }
//+------------------------------------------------------------------+
//| Moving Averages Convergence/Divergence                           |
//+------------------------------------------------------------------+
int start()
  {
   int limit,i;
   int counted_bars=IndicatorCounted();
//---- check for possible errors
   if(counted_bars<1)
     {
      for(i=1;i<=draw_begin0;i++) ind_buffer0[Bars-i]=0;
      for(i=1;i<=HMA_Period;i++) ind_buffer1[Bars-i]=0;
     }
//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;
//---- MA difference counted in the 1-st buffer
   for(i=0; i<limit; i++)
   {
      double v1 = iMA(NULL,0,MathFloor(HMA_Period/2),0,MODE_LWMA,Price,i);
      double v2 = iMA(NULL,0,HMA_Period,0,MODE_LWMA,Price,i);
      ind_buffer1[i]=2.0*v1-v2;
   }                      
                      
//---- HMA counted in the 0-th buffer
   for(i=limit; i>=0; i--)
   {
      ind_buffer0[i]=iMAOnArray(ind_buffer1,0,MathFloor(MathSqrt(HMA_Period)),0,MODE_LWMA,i);
      
      if (ColorMode>0)
      {
         trend[i] = trend[i+1];
         if (ind_buffer0[i]> ind_buffer0[i+1]) trend[i] = 1;
         if (ind_buffer0[i]< ind_buffer0[i+1]) trend[i] =-1;
    
         if (trend[i]>0)
         {
            Uptrend[i] = ind_buffer0[i]; 
            if (trend[i+ColorBarBack]<0) Uptrend[i+ColorBarBack]=ind_buffer0[i+ColorBarBack];
            Dntrend[i] = EMPTY_VALUE;
            if (WarningMode>0 && trend[i+1]<0 && i==0) PlaySound("alert2.wav");
         }
         else              
         if (trend[i]<0)
         { 
            Dntrend[i] = ind_buffer0[i]; 
            if (trend[i+ColorBarBack]>0) Dntrend[i+ColorBarBack]=ind_buffer0[i+ColorBarBack];
            Uptrend[i] = EMPTY_VALUE;
            if (WarningMode>0 && trend[i+1]>0 && i==0) PlaySound("alert2.wav");
         }               
      }
   }      
//----------   
   string Message;
   
   if ( trend[2]<0 && trend[1]>0 && Volume[0]>1 && !UpTrendAlert)
	{
	Message = " "+Symbol()+" M"+Period()+": HMA Signal for BUY";
	if ( AlertMode>0 )Alert (Message);
	if ( EmailOn > 0) SendMail("HMA  alert","Buy signal at Ask="+DoubleToStr(Ask,4)+", Bid="+DoubleToStr(Bid,4)+", Date="+TimeToStr(CurTime(),TIME_DATE)+" "+TimeHour(CurTime())+":"+TimeMinute(CurTime())+" Symbol="+Symbol()+" Period="+Period());
   if ( SoundOn > 0) PlaySound("pirate.wav");
	UpTrendAlert=true; DownTrendAlert=false;
	} 
	 	  
	if ( trend[2]>0 && trend[1]<0 && Volume[0]>1 && !DownTrendAlert)
	{
	Message = " "+Symbol()+" M"+Period()+": HMA Signal for SELL";
	if ( AlertMode>0 ) Alert (Message);
	if ( EmailOn > 0 ) SendMail("HMA  alert","Sell signal at Ask="+DoubleToStr(Ask,4)+", Bid="+DoubleToStr(Bid,4)+", Date="+TimeToStr(CurTime(),TIME_DATE)+" "+TimeHour(CurTime())+":"+TimeMinute(CurTime())+" Symbol="+Symbol()+" Period="+Period());
   if ( SoundOn  > 0) PlaySound("pirate.wav"); 
	DownTrendAlert=true; UpTrendAlert=false;
	} 	         



//---- done
   return(0);
  }